package com.adonai.manman.parser;

import android.support.annotation.NonNull;

/**
 * Helper class to deal with GROFF special characters
 *
 * @author Adonai
 */
public class SpecialsHandler {

    /**
     * Returns appropriate UTF-8 character for requested GROFF special code
     * @param code code to look for
     * @return string of one  UTF-8 character (can't use char here, they're 2-byte almost always)
     */
    @NonNull
    static String parseSpecial(String code) {
        switch (code) {
            case "-D":
                return "Ð";
            case "Sd":
                return "ð";
            case "TP":
                return "Þ";
            case "Tp":
                return "þ";
            case "ss":
                return "ß";
            case "AE":
                return "Æ";
            case "ae":
                return "æ";
            case "OE":
                return "Œ";
            case "oe":
                return "œ";
            case ".i":
                return "ı";
            case "'A":
                return "Á";
            case "'E":
                return "É";
            case "'I":
                return "Í";
            case "'O":
                return "Ó";
            case "'U":
                return "Ú";
            case "'Y":
                return "Ý";
            case "'a":
                return "á";
            case "'e":
                return "é";
            case "'i":
                return "í";
            case "'o":
                return "ó";
            case "'u":
                return "ú";
            case "'y":
                return "ý";
            case ":A":
                return "Ä";
            case ":E":
                return "Ë";
            case ":I":
                return "Ï";
            case ":O":
                return "Ö";
            case ":U":
                return "Ü";
            case ":Y":
                return "Ÿ";
            case ":a":
                return "ä";
            case ":e":
                return "ë";
            case ":i":
                return "ï";
            case ":o":
                return "ö";
            case ":u":
                return "ü";
            case ":y":
                return "ÿ";
            case "^A":
                return "Â";
            case "^E":
                return "Ê";
            case "^I":
                return "Î";
            case "^O":
                return "Ô";
            case "^U":
                return "Û";
            case "^a":
                return "â";
            case "^e":
                return "ê";
            case "^i":
                return "î";
            case "^o":
                return "ô";
            case "^u":
                return "û";
            case "`A":
                return "À";
            case "`E":
                return "È";
            case "`I":
                return "Ì";
            case "`O":
                return "Ò";
            case "`U":
                return "Ù";
            case "`a":
                return "à";
            case "`e":
                return "è";
            case "`i":
                return "ì";
            case "`o":
                return "ò";
            case "`u":
                return "ù";
            case "~A":
                return "Ã";
            case "~N":
                return "Ñ";
            case "~O":
                return "Õ";
            case "~a":
                return "ã";
            case "~n":
                return "ñ";
            case "~o":
                return "õ";
            case "vS":
                return "Š";
            case "vs":
                return "š";
            case "vZ":
                return "Ž";
            case "vz":
                return "ž";
            case ",C":
                return "Ç";
            case ",c":
                return "ç";
            case "/L":
                return "Ł";
            case "/l":
                return "ł";
            case "/O":
                return "Ø";
            case "/o":
                return "ø";
            case "oA":
                return "Å";
            case "oa":
                return "å";
            case "a\"":
                return "\"";
            case "a-":
                return "¯";
            case "a.":
                return "˙";
            case "a^":
                return "^";
            case "aa":
                return "'";
            case "ga":
                return "`";
            case "ab":
                return "˘";
            case "ac":
                return "¸";
            case "ad":
                return "¨";
            case "ah":
                return "ˇ";
            case "ao":
                return "˚";
            case "a~":
                return "~";
            case "ho":
                return "˛";
            case "ha":
                return "^";
            case "ti":
                return "∼";
            case "Bq":
                return "\"";
            case "bq":
                return "'";
            case "lq":
                return "\"";
            case "rq":
                return "\"";
            case "oq":
                return "'";
            case "cq":
                return "’";
            case "aq":
                return "‘";
            case "dq":
                return "\"";
            case "Fo":
                return "«";
            case "Fc":
                return "»";
            case "fo":
                return "‹";
            case "fc":
                return "›";
            case "r!":
                return "¡";
            case "r?":
                return "¿";
            case "em":
                return "-";
            case "en":
                return "-";
            case "hy":
                return "-";
            case "lB":
                return "[";
            case "rB":
                return "]";
            case "lC":
                return "{";
            case "rC":
                return "}";
            case "la":
                return "<";
            case "ra":
                return ">";
            case "<-":
                return "←";
            case "->":
                return "→";
            case "<>":
                return "↔";
            case "da":
                return "↓";
            case "ua":
                return "⇑";
            case "lA":
                return "⇐";
            case "rA":
                return "⇒";
            case "hA":
                return "⇔";
            case "dA":
                return "⇓";
            case "uA":
                return "⇑";
            case "an":
                return "─";
            case "or":
                return "|";
            case "ba":
                return "|";
            case "br":
                return "│";
            case "ru":
                return "_";
            case "ul":
                return "_";
            case "bv":
                return "│";
            case "bb":
                return "¦";
            case "sl":
                return "/";
            case "rs":
                return "\\";
            case "ci":
                return "◯";
            case "bu":
                return "•";
            case "dd":
                return "‡";
            case "dg":
                return "†";
            case "lz":
                return "◊";
            case "sq":
                return "□";
            case "ps":
                return "¶";
            case "sc":
                return "§";
            case "lh":
                return "☜";
            case "rh":
                return "☞";
            case "at":
                return "@";
            case "sh":
                return "#";
            case "CR":
                return "␍";
            case "co":
                return "©";
            case "rg":
                return "®";
            case "tm":
                return "™";
            case "Do":
                return "$";
            case "ct":
                return "¢";
            case "eu":
                return "€";
            case "Eu":
                return "€";
            case "Ye":
                return "¥";
            case "Po":
                return "£";
            case "Cs":
                return "¤";
            case "Fn":
                return "ƒ";
            case "de":
                return "°";
            case "%0":
                return "‰";
            case "fm":
                return "′";
            case "sd":
                return "″";
            case "mc":
                return "µ";
            case "Of":
                return "ª";
            case "Om":
                return "º";
            case "AN":
                return "∧";
            case "OR":
                return "∨";
            case "no":
                return "¬";
            case "te":
                return "∃";
            case "fa":
                return "∀";
            case "st":
                return "∋";
            case "3d":
                return "∴";
            case "tf":
                return "∴";
            case "12":
                return "½";
            case "14":
                return "¼";
            case "34":
                return "¾";
            case "S1":
                return "¹";
            case "S2":
                return "²";
            case "S3":
                return "³";
            case "pl":
                return "+";
            case "+-":
                return "±";
            case "t+-":
                return "±";
            case "pc":
                return "·";
            case "md":
                return "⋅";
            case "mu":
                return "×";
            case "tmu":
                return "×";
            case "c*":
                return "⊗";
            case "c+":
                return "⊕";
            case "di":
                return "÷";
            case "tdi":
                return "÷";
            case "f/":
                return "⁄";
            case "**":
                return "∗";
            case "<=":
                return "≤";
            case ">=":
                return "≥";
            case "!=":
                return "≠";
            case "eq":
                return "=";
            case "==":
                return "≡";
            case "=~":
                return "≅";
            case "ap":
                return "∼";
            case "~~":
                return "≈";
            case "~=":
                return "≈";
            case "pt":
                return "∝";
            case "es":
                return "∅";
            case "mo":
                return "∈";
            case "nm":
                return "∉";
            case "nb":
                return "⊄";
            case "sb":
                return "⊂";
            case "sp":
                return "⊃";
            case "ib":
                return "⊆";
            case "ip":
                return "⊇";
            case "ca":
                return "∩";
            case "cu":
                return "∪";
            case "/_":
                return "∠";
            case "pp":
                return "⊥";
            case "is":
                return "∫";
            case "sum":
                return "∑";
            case "product":
                return "∏";
            case "gr":
                return "∇";
            case "sr":
                return "√";
            case "rn":
                return "‾";
            case "if":
                return "∞";
            case "Ah":
                return "ℵ";
            case "Im":
                return "ℑ";
            case "Re":
                return "ℜ";
            case "wp":
                return "℘";
            case "pd":
                return "∂";
            case "*A":
                return "Α";
            case "*B":
                return "Β";
            case "*C":
                return "Ξ";
            case "*D":
                return "Δ";
            case "*E":
                return "Ε";
            case "*F":
                return "Φ";
            case "*G":
                return "Γ";
            case "*H":
                return "Θ";
            case "*I":
                return "Ι";
            case "*K":
                return "Κ";
            case "*L":
                return "Λ";
            case "*M":
                return "Μ";
            case "*N":
                return "Ν";
            case "*O":
                return "Ο";
            case "*P":
                return "Π";
            case "*Q":
                return "Ψ";
            case "*R":
                return "Ρ";
            case "*S":
                return "Σ";
            case "*T":
                return "Τ";
            case "*U":
                return "Υ";
            case "*W":
                return "Ω";
            case "*X":
                return "Χ";
            case "*Y":
                return "Η";
            case "*Z":
                return "Ζ";
            case "*a":
                return "α";
            case "*b":
                return "β";
            case "*c":
                return "ξ";
            case "*d":
                return "δ";
            case "*e":
                return "ε";
            case "*f":
                return "φ";
            case "+f":
                return "ϕ";
            case "*g":
                return "γ";
            case "*h":
                return "θ";
            case "+h":
                return "ϑ";
            case "*i":
                return "ι";
            case "*k":
                return "κ";
            case "*l":
                return "λ";
            case "*m":
                return "μ";
            case "*n":
                return "ν";
            case "*o":
                return "ο";
            case "*p":
                return "π";
            case "+p":
                return "ϖ";
            case "*q":
                return "ψ";
            case "*r":
                return "ρ";
            case "*s":
                return "σ";
            case "*t":
                return "τ";
            case "*u":
                return "υ";
            case "*w":
                return "ω";
            case "*x":
                return "χ";
            case "*y":
                return "η";
            case "*z":
                return "ζ";
            case "ts":
                return "ς";
            case "CL":
                return "♣";
            case "SP":
                return "♠";
            case "HE":
                return "♥";
            case "DI":
                return "♦";
            case "mi":
                return "-";
            default:
                return "";
        }
    }
}
